function [ f_ion ] = compute_ion_spectrum_from_LEDA_electron_spectrum( p, f, w_e1, w_e2, alpha_e, dpixel, Gpgc, SN )
% -------------------------------------------------------------------------
%
% NAME
%
%	compute_ion_spectrum_from_LEDA_electron_spectrum
%
% PURPOSE
%
%	This routine computes the ion spectrum incident on the MCP from
%   the electron spectrum recorded by the LEDA, knowing the
%   electron cascade properties and the position-dependent MCP gain.
%
% CALLING SEQUENCE
%
%	[ f_ion ] = compute_ion_spectrum_from_LEDA_electron_spectrum( p, f, w_e1, w_e2, alpha_e, dpixel, Gpgc, SN )
%
% INPUT PARAMETERS
%   
%   p
%     pixel vector
%
%   f
%     electron spectrum recorded by the LEDA
%
%   w_e1, w_e2, alpha_e
%     half-widths of both Gaussian components and relative importance of
%     the secondary Gaussian in the electron cascade
%
%   dpixel
%     half-width in pixels of the spread due to misalignment of analysis
%     direction and LEDA row orientation
%
%   Gpgc
%     position-dependent gain factor
%     double, column vector with n elements
%
%   SN
%     signal to noise ratio
%
% OUTPUT PARAMETERS
%
%   f_ion
%     recovered ion spectrum, same size as p
%
% AUTHOR
%
%   (c) Johan De Keyser - Royal Belgian Institute for Space Aeronomy - 2019
%
% HISTORY
%
%   2019-05-15 : v0.0
%     Initial version
%   2019-06-15 : v1.0
%     Included misalignment effects
%   2019-06-26 : v1.1
%     Choice between single- and two-grid iterative solvers
%   2019-07-31 : v1.2
%     Replaced by a direct solver
%
% -------------------------------------------------------------------------
            
    [ f_ion, F ] = ...
        compute_ion_spectrum_from_LEDA_electron_spectrum_core( ...
            p, f, ...
            w_e1, w_e2, alpha_e, ...
            dpixel, Gpgc, ...
            SN ...
        );
    fprintf( '%10.6f %10.6f %10.6f : %12.8f ***\n', w_e1, w_e2, alpha_e, F );       
                    
end

% =========================================================================
    
function [ f_ion, F ] = compute_ion_spectrum_from_LEDA_electron_spectrum_core( p, f, w_e1, w_e2, alpha_e, dpixel, Gpgc, SN )
% -------------------------------------------------------------------------
%
% NAME
%
%	compute_ion_spectrum_from_LEDA_electron_spectrum_core
%
% PURPOSE
%
%	This routine computes the ion spectrum incident on the MCP from
%   the electron spectrum recorded by the LEDA, knowing the
%   electron cascade properties and the position-dependent MCP gain.
%
% CALLING SEQUENCE
%
%	[ f_ion, F ] = compute_ion_spectrum_from_LEDA_electron_spectrum_core( p, f, w_e1, w_e2, alpha_e, dpixel, Gpgc, f_ion_0, zeta, SN )
%
% INPUT PARAMETERS
%   
%   p
%     pixel vector
%
%   f
%     electron spectrum recorded by the LEDA
%
%   w_e1, w_e2, alpha_e
%     half-widths of both Gaussian components and relative importance of
%     the secondary Gaussian in the electron cascade
%
%   dpixel
%     half-width in pixels of the spread due to misalignment of analysis
%     direction and LEDA row orientation
%
%   Gpgc
%     position-dependent gain factor
%     double, column vector with n elements
%
%   SN
%     signal to noise ratio
%
% OUTPUT PARAMETERS
%
%   f_ion
%     recovered ion spectrum, same size as p
%
%   F
%     fitting quality
%
% AUTHOR
%
%   (c) Johan De Keyser - Royal Belgian Institute for Space Aeronomy - 2019
%
% HISTORY
%
%   2019-05-15 : v0.0
%     Initial version
%   2019-06-26 : v0.0
%     Implemented multi-grid version with some acceleration features
%   2019-07-31 : v1.0
%     Replaced by direct solver
%
% -------------------------------------------------------------------------
    
    % smoothing factor
    gamma = 0.0001;
    
    % number of pixels
    np = length(p);
    dp = p(2) - p(1);
    dp2 = dp^2;

    % compute template
    pref = 0.5*(p(1)+p(end));
    f_cascade = electron_cascade( p, pref, w_e1, w_e2, alpha_e, dpixel );
    range = ( f_cascade > 1e-16 );
    p_minus_pref = p(range) - pref;
    f_cascade = f_cascade(range);
    
    % compute system matrix
    ZZ = zeros( [ np, np ] );
    Psi = ZZ;
    for s = 1:np
        f_electron = ...
            interp1( p_minus_pref, f_cascade, p-p(s), 'pchip', 1e-99 );
        Psi(s,:) = f_electron(:);
    end
    Psi( Psi < 1e-16 ) = 0;
    % Psi = sparse( Psi );
    Psi_Gpgc = Psi .* Gpgc;

    % Build least-squares problem 
    C = sparse( ...
            [ 2:np-1, 2:np-1, 2:np-1 ], ...
            [ 2:np-1, 1:np-2, 3:np ], ...
            [ 2*ones(1,np-2), -ones(1,np-2), -ones(1,np-2) ], ...
            np, np, 3*(np-2) ...
        ) / dp2;
    PsiPsi = dp2 * ( Psi_Gpgc * Psi_Gpgc' );
    CC = ( C * C' );
    diag_Gpgc_Psi = diag(Gpgc) * Psi;
    b = diag_Gpgc_Psi * f;
    
    % solve system - direct solution
    
    % prepare for trials with a range of smoothing values
    n_try_half = 3;
    n_try = 2*n_try_half + 1;
    A = cell( 1, n_try );
    Ainv = cell( 1, n_try );
    Inp = eye(np);
    factor = 10.^linspace(-2,2,n_try);

    % first build all the matrices and inverses
    for k = -n_try_half:1:n_try_half
        index = k+n_try_half+1;
        A{index} = PsiPsi + factor(index) * gamma * CC;
        Ainv{index} = A{index} \ Inp;
    end

    % downselect to what is useful
    useful_range = ( b > 1e-90 );
    bu = b(useful_range);            

    f_ion_all = zeros( np, n_try );
    for k = -n_try_half:1:n_try_half
        index = k+n_try_half+1;
        Auinv = Ainv{index}(useful_range,useful_range);
        f_ion_u = Auinv * bu;
        f_ion_u(f_ion_u < 1e-99) = 1e-99;
        f_ion = 0*b;
        f_ion(useful_range) = f_ion_u;
        f_ion_all( :, index ) = f_ion;
    end
    f_ion = f_ion_all(:,n_try_half+1);
    the_max = max(f_ion);
    the_sum = sum(f_ion);
    mask = ( min( f_ion_all, [], 2 ) > the_max / SN );
    f_ion(~mask) = 1e-99;
    the_sum_corrected = sum(f_ion);
    f_ion = ( the_sum / the_sum_corrected ) * f_ion;

    A = A{n_try_half+1};
    
    mask = (f > 1e-16);
    A_f_ion = A*f_ion;
    F = sqrt( sum( ( ( A_f_ion(mask)-b(mask) )./sqrt(b(mask)) ).^2 ) / np ); 

end

% =========================================================================
