function [ f ] = create_LEDA_electron_spectrum( p, f_ion, w_e1, w_e2, alpha_e, dpixel, Gpgc )
% -------------------------------------------------------------------------
%
% NAME
%
%	create_LEDA_electron_spectrum
%
% PURPOSE
%
%	This routine creates a synthetic LEDA electron spectrum for a given
%   set of incoming ions, with the prescribed ion peak parameters and
%   electron cascade properties, and with the position-dependent MCP gain.
%
% CALLING SEQUENCE
%
%	[ f ] = create_LEDA_electron_spectrum( p, f_ion, w_e1, w_e2, alpha_e, dpixel, Gpgc )
%
% INPUT PARAMETERS
%   
%   p
%     pixel vector
%
%   f_ion
%     ion spectrum
%
%   w_e1, w_e2, alpha_e
%     half-widths of both Gaussian components and relative importance of
%     the secondary Gaussian in the electron cascade
%
%   dpixel
%     half-width in pixels of the spread due to misalignment of analysis
%     direction and LEDA row orientation
%
%   Gpgc
%     position-dependent gain factor
%
% OUTPUT PARAMETERS
%
%   f
%     synthetic LEDA electron spectrum, same format as p
%
% AUTHOR
%
%   (c) Johan De Keyser - Royal Belgian Institute for Space Aeronomy - 2019
%
% HISTORY
%
%   2019-05-15 : v0.0
%     Initial version
%   2019-06-18 : v1.0
%     Incorporated the effect of misalignment
%
% -------------------------------------------------------------------------
    
    % Number of pixels
    np = length(p);
    dp = ( p(2)-p(1) );
    dp2 = dp^2;
        
    % compute template
    pref = 0.5*(p(1)+p(end));
    f_cascade = electron_cascade( p, pref, w_e1, w_e2, alpha_e, dpixel );
    range = ( f_cascade > 1e-16 );
    p_minus_pref = p(range) - pref;
    f_cascade = f_cascade(range);
    
    % Add electron cascades
    f = 0*p;
    f_ion_max = max(f_ion);
    f_ion_limit = 1e-16 * f_ion_max;
    for i = 1:np
        if f_ion(i) > f_ion_limit
            % At each position along the misaligned ion analysis direction, use
            % the correct position-dependent Gpgc
            f_electron = ...
                interp1( p_minus_pref, f_cascade, p-p(i), 'pchip', 1e-99 );
            f = f + ( f_ion(i) * Gpgc(i) ) * f_electron;
        end
    end
    f = dp2 * f;
        
end

% =========================================================================
