function [ f_electron ] = electron_cascade( p, pstar, w1, w2, alpha, dpixel )
% -------------------------------------------------------------------------
%
% NAME
%
%	electron_cascade
%
% PURPOSE
%
%	This routine creates a double Gaussian electron cascade shape with
%   broadening due to misalignment, with the prescribed parameters.
%
% CALLING SEQUENCE
%
%	[ f_electron ] = electron_cascade( p, pstar, w1, w2, alpha, dpixel )
%
% INPUT PARAMETERS
%   
%   p
%     pixel vector
%
%   pstar
%     position of the centre of the electron cascade
%
%   w1, w2, alpha
%     half-widths of both Gaussian components and relative importance of
%     the secondary Gaussian
%
%   dpixel
%     half-width in pixels of the spread due to misalignment of analysis
%     direction and LEDA row orientation
%
% OUTPUT PARAMETERS
%
%   f_electron
%     electron cascade shape, same format as p
%
% AUTHOR
%
%   (c) Johan De Keyser - Royal Belgian Institute for Space Aeronomy - 2019
%
% HISTORY
%
%   2019-05-15 : v0.0
%     Initial version
%   2019-06-18 : v1.0
%     Incorporated the effect of misalignment
%
% -------------------------------------------------------------------------

    % Create electron cascade, normalized to unit area
    
    % To model the effect of misalignment, we will add a number of double
    % gaussians, shifted over a series of offsets. For sufficient
    % precision, adding double gaussians shifted over 0.1 pixel seems
    % good enough.
    if dpixel > 0
        n_substeps = 2*ceil( dpixel/0.1 );
    else
        n_substeps = 1;
    end
    offset = linspace(-dpixel,dpixel,n_substeps);
    
    % Compute double gaussian template
    pref = 0.5*(p(1)+p(end));
    dg = double_gaussian( p, pref, w1, w2, alpha ) + 1e-99;
    
    % Determine range of interest
    np = length(p);
    idx_mid = round( np/2 );
    ceil_dpixel = ceil(dpixel);
    n_significant = ceil( sum( dg > 1e-16 ) / 2 );
    range = ( idx_mid-n_significant-ceil_dpixel : idx_mid+n_significant+ceil_dpixel );
    p_minus_pref = p(range) - pref;
    dg = dg(range);
    
    % Add together
    f_electron = 0*p;
    p_minus_pstar = p - pstar;
    for i = 1:n_substeps
        the_f_electron = interp1( p_minus_pref, dg, p_minus_pstar-offset(i), 'pchip', 1e-99 );
        f_electron = ...
            f_electron ...
            + ...
            the_f_electron;
    end 
    
    % Normalize to unit area
    f_electron = ...
        f_electron ...
        / ...
        ( sum(f_electron) * (p(2)-p(1)) );

end

% =========================================================================
